@@ -53,13 +53,12 @@ use coap_handler::Handler;
 
 use coap_numbers::{code, option};
 
-// Used to store response codes internally until the type of the response message is determined and
-// it can be converted into an actual outgoing code
-//
-// FIXME One way to do this better might be to make the handler already specific to the response
-// message type, then the code could be statically converted into whichever code type that uses
-// already at extraction time. (Another option would be to have an enum here with the codes we know
-// we use, and limit outputs to those that implement an From<OutCode>)
+/// Used to store response codes internally until the type of the response message is determined and
+/// it can be converted into an actual outgoing code
+///
+/// It was tried to use an enum here, but in the end that needs to go through something
+/// [codeconvert]-like just again. As the output message type is generally not known in advance,
+/// that can't be used either.
 type Code = u8;
 
 /// A resource that unconditionally responds 4.04 Not Found.
@@ -274,14 +273,14 @@ pub struct SimpleRenderableData(Result<Block2RequestData, Code>);
 /// [SimpleRendered] to form a Handler.
 ///
 /// For binary data, render_bytes can be implemented instead of render, and utilize the
-/// BlockWriter's write_bytes method (multiple times, if so desired). In that case, the render
+/// WindowedInfinityWithETag's write_bytes method (multiple times, if so desired). In that case, the render
 /// function can be left unimplemented.
 pub trait SimpleRenderable {
     fn render<W>(&mut self, writer: &mut W)
     where
         W: core::fmt::Write;
 
-    fn render_bytes(&mut self, writer: &mut helpers::BlockWriter)
+    fn render_bytes(&mut self, writer: &mut helpers::WindowedInfinityWithETag)
     {
         self.render(writer)
     }
@@ -400,9 +399,8 @@ impl<'a> SimpleRenderable for TypedStaticRenderable<'a> {
         unimplemented!("render_bytes is implemented instead")
     }
 
-    fn render_bytes(&mut self, writer: &mut helpers::BlockWriter) {
-        writer
-            .write_bytes(self.data)
+    fn render_bytes(&mut self, writer: &mut helpers::WindowedInfinityWithETag) {
+        writer.write(self.data)
     }
 
     fn content_format(&self) -> Option<u16> {
@@ -411,7 +409,6 @@ impl<'a> SimpleRenderable for TypedStaticRenderable<'a> {
 }
 
 use serde::Serialize;
-use serde_cbor;
 
 /// A simple Handler trait that supports GET, POST and/or PUT on a data structure that supports
 /// serde.
@@ -457,7 +454,9 @@ impl<H: SimpleCBORHandler> SimpleCBORWrapper<H> {
         SimpleCBORWrapper { handler }
     }
 
-    fn check_get_options(request: &impl ReadableMessage) -> Result<(), Code> {
+    fn check_get_options(request: &impl ReadableMessage) -> Result<Block2RequestData, Code> {
+        let mut block2 = Ok(None);
+
         for o in request.options() {
             match o.number() {
                 option::ACCEPT => {
@@ -465,6 +464,15 @@ impl<H: SimpleCBORHandler> SimpleCBORWrapper<H> {
                         return Err(code::UNSUPPORTED_CONTENT_FORMAT);
                     }
                 }
+                option::BLOCK2 => {
+                    block2 = match block2 {
+                        Err(e) => Err(e),
+                        Ok(Some(_)) => Err(coap_numbers::code::BAD_REQUEST),
+                        Ok(None) => Block2RequestData::from_option(&o)
+                            .map(Some)
+                            .map_err(|()| code::BAD_REQUEST),
+                    }
+                }
                 o if option::get_criticality(o) == option::Criticality::Critical => {
                     return Err(code::BAD_OPTION);
                 }
@@ -472,7 +480,7 @@ impl<H: SimpleCBORHandler> SimpleCBORWrapper<H> {
             }
         }
 
-        Ok(())
+        Ok(block2?.unwrap_or_default())
     }
 
     fn check_postput_options(request: &impl ReadableMessage) -> Result<(), Code> {
@@ -495,7 +503,7 @@ impl<H: SimpleCBORHandler> SimpleCBORWrapper<H> {
 }
 
 pub enum SimpleCBORRequestData {
-    Get,        // Nothing has been processed, decision deferred to build_response stage
+    Get(Block2RequestData), // GET to be processed later, but all request opions were in order
     Done(Code), // All done, just a response to emit -- if POST/PUT has been processed, or GET had a bad accept/option
 }
 use self::SimpleCBORRequestData::{Done, Get};
@@ -511,11 +519,10 @@ where
     fn extract_request_data(&mut self, request: &impl ReadableMessage) -> Self::RequestData {
         match request.code().into() {
             code::GET => {
-                if let Err(e) = Self::check_get_options(request) {
-                    return Done(e);
+                match Self::check_get_options(request) {
+                    Err(e) => Done(e),
+                    Ok(block) => Get(block),
                 }
-
-                Get
             }
             code::POST => {
                 if let Err(e) = Self::check_postput_options(request) {
@@ -550,7 +557,7 @@ where
     fn estimate_length(&mut self, request: &Self::RequestData) -> usize {
         match request {
             Done(_) => 4,
-            Get => 1280 - 40 - 4, // does this correclty calculate the IPv6 minimum MTU?
+            Get(block) => (block.size() + 25).into(), // FIXME: hard-coded copied over from block2_write_with_cf's estimated overhead
         }
     }
 
@@ -564,7 +571,7 @@ where
                 response.set_code(codeconvert(r));
                 response.set_payload(b"");
             }
-            Get => {
+            Get(block2) => {
                 let repr = self.handler.get();
                 match repr {
                     Err(e) => {
@@ -573,35 +580,16 @@ where
                     }
                     Ok(repr) => {
                         response.set_code(codeconvert(code::CONTENT));
-                        // FIXME: This needs to be rewound on overly-long errors
-                        if let Ok(cfopt) = option::CONTENT_FORMAT.try_into() {
-                            response.add_option_uint(cfopt, 60u8);
-                        }
-
-                        let (written, available) = {
-                            // FIXME An API that just says how much payload I may use would be
-                            // better here.
-                            let len = response.available_space() - 1;
-                            // FIXME This is a bad estimate; enhance using better knowledge of CBOR
-                            // encoding or just go full block-wise. (Trimming at 1KiB payload
-                            // because the usize::MAX unbounded impls would yield is clearly too
-                            // much)
-                            let len = len.min(1024);
-                            let payload = &mut response.payload_mut_with_len(len);
-                            let mut win = windowed_infinity::WindowedInfinity::new(payload, 0);
-
-                            match repr.serialize(&mut serde_cbor::ser::Serializer::new(&mut win)) {
-                                Ok(()) => (win.get_cursor().try_into().ok(), payload.len()),
-                                Err(_) => (None, 0),
-                            }
-                        };
-                        match written {
-                            Some(w) if w <= available => {
-                                response.truncate(w);
-                            }
-                            _ => {
+                        match block2_write_with_cf(
+                            block2,
+                            response,
+                            |win| repr.serialize(&mut serde_cbor::ser::Serializer::new(win)),
+                            Some(60),
+                        ) {
+                            Ok(()) => (),
+                            Err(_) => {
+                                // FIXME: Rewind all the written options
                                 response.set_code(codeconvert(code::INTERNAL_SERVER_ERROR));
-                                response.truncate(0);
                             }
                         }
                     }
